//
//  VPNUConfigurator.h
//  VPNUnlimitedSDK
//
//  Copyright (c) 2015 KeepSolid. All rights reserved.
//

//
// Use of this SDK is subject to the VPN Unlimited SDK License Agreement
// https://www.keepsolid.com/static/docs/VPN_Unlimited_SDK_License.html
//

#import <Foundation/Foundation.h>
#import <VPNUnlimitedSDK/VPNUConstants.h>

/*!
 * @typedef VPNUConnectionStatus
 * @abstract VPN status codes
 */
typedef NS_ENUM(NSInteger, VPNUConnectionStatus) {
    /*! @const VPNUConnectionStatusInvalid The VPN is not configured. */
    VPNUConnectionStatusInvalid = 0,
    /*! @const VPNUConnectionStatusDisconnected The VPN is disconnected. */
    VPNUConnectionStatusDisconnected = 1,
    /*! @const VPNUConnectionStatusConnecting The VPN is connecting. */
    VPNUConnectionStatusConnecting = 2,
    /*! @const VPNUConnectionStatusConnected The VPN is connected. */
    VPNUConnectionStatusConnected = 3,
    /*! @const VPNUConnectionStatusReasserting The VPN is reconnecting following loss of underlying network connectivity. */
    VPNUConnectionStatusReasserting = 4,
    /*! @const VPNUConnectionStatusDisconnecting The VPN is disconnecting. */
    VPNUConnectionStatusDisconnecting = 5,
} NS_ENUM_AVAILABLE(10_7, 8_0);

@class VPNUTransport;
@class VPNUServerItem;
@class VPNUOnDemandConnectionManager;
@class VPNULocalServer;

/*!
 @class VPNUConfigurator
 @abstract
 This class provides an interface for controlling VPN connections.
 @discussion
 Before using this class you must authorize or register with the help of @c VPNUAuthorizer class. You must pass the same @c VPNUTransport instance to its constructor as you have passed to @c VPNUAuthorizer instance constructor.
 
 A basic workflow after initializing looks as follows:
 For iOS 9+ / macOS 10.11+ only (ignore first two steps if you don't support iOS 9+ / macOS 10.11+):
 
 1. Set desired value for `protocol` property if needed.
 2. Asynchronously call @c loadInitialConfiguration.
 
 3. Provide a custom @c VPNUOnDemandConnectionManager object for @c onDemandConnectionManager property if you want to manage VPN On Demand Rules.
 4. Configure VPN server item using @c setupWithServerItem:alreadyInstalled: method.
 5. Connect to the configured VPN server using @c startVPNWithError: method.
 6. Disconnect at any time using @c stopVPNWithError: method.
 
 
 You can check @c connectionStatus property of this class to know the real VPN connection status every time @c VPNUConnectionStatusDidChangeNotification notification is posted or simply use KVO.
 
 @attention This class is a public interface of a class cluster. You are discouraged from subclassing it.
 @attention You MUST call @c loadInitialConfiguration method asynchronously if you are using iOS 9+ / macOS 10.11+ BEFORE any call to instance of this class.
 */
NS_AVAILABLE(10_7, 7_0)
@interface VPNUConfigurator : NSObject

/*!
 @brief Protocol used when downloading and configuring VPN profile.
 @discussion Set this property before calling @c setupWithServerItem:alreadyInstalled: method. Default value is @c VPNUProtocolIPSec.
 */
@property VPNUProtocol protocol;

/*!
 @brief Application scheme used by local server.
 */
@property (copy) NSString *appScheme;

/*!
 @brief Custom name for a downloaded VPN profile.
 */
@property (copy) NSString *profileName;

/*!
 @brief Custom name for a downloaded VPN profile if Wise or OpenVPN is selected as protocol.
 
 @discussion If not set - `profileName` is used.
 */
@property (copy) NSString *customProtocolProfileName NS_AVAILABLE_MAC(10_11);

/*!
 @brief An absolute path to your custom local HTML page which is shown to user during profile download process.
 */
@property (nonatomic, copy) NSString *customProfileDownloadHTMLPath NS_AVAILABLE_MAC(10_7);

/*! 
 @brief This property is set to @c YES if VPN configuration for current app is enabled.
 @discussion This property will be set to @c NO when other VPN configurations (from other apps) are enabled.
 */
@property (readonly, getter=isVPNEnabled) BOOL VPNEnabled NS_AVAILABLE(10_11, 8_0);

/*!
 @brief If you want to manage VPN On Demand Rules using @c VPNUOnDemandConnectionManager, set its instance here.
 @discussion If you don't set this value before configuring a new connection with @c setupWithServerItem: default On Demand rules will be applied for the connection.
 */
@property (strong) VPNUOnDemandConnectionManager *onDemandConnectionManager NS_AVAILABLE(10_7, 8_0);

/*!
@abstract
Tells the configurator to use @c NEVPNManager class for configuring and controlling a VPN.
@discussion
If set to @c YES, user will be able to turn a VPN connection on and off inside the host application. If set to @c NO, user will have to explicitly download profiles and turn a VPN connection on and off inside iOS Settings (pre-iOS8 style). Default value is @c YES.
*/
@property BOOL useNEVPNManager NS_AVAILABLE(10_11, 8_0);
/// The current status of the VPN connection. This value is KVO-compliant.
@property (readonly) VPNUConnectionStatus connectionStatus NS_AVAILABLE(10_7, 7_0);

/*!
 @brief Loads VPN manager for selected protocol.
 @discussion Call this method asynchronously to load default VPN manager for selected protocol if you are using this class on iOS 9+ / macOS 10.11 and support custom protocol.
 @warning This method shouldn't be called from main thread.
 @return @c YES if load was successful, @c NO otherwise.
 */
- (BOOL)loadInitialConfiguration NS_AVAILABLE(10_11, 9_0);

/*!
 @brief Installs VPN profile for the specified server item.
 @discussion This method first retrieves VPN profile information for the specified server item. If this step was succesful, it then tries to install a new VPN profile into the system using the information retrieved in the first step. If configuration downloaded for the server item is corrupted (e.g., has no server address) the profile is automatically deleted. This method blocks a calling thread.
 @warning This method shouldn't be called from main thread.
 @param serverItem Server item object to perform setup for. Cannot be @c nil.
 @param alreadyInstalled This value is set to @c YES, if VPN profile is already installed.
 @return @c nil if setup was successful, error object otherwise.
 */
- (NSError *)setupWithServerItem:(VPNUServerItem *)serverItem alreadyInstalled:(BOOL *)alreadyInstalled NS_AVAILABLE(10_7, 7_0);
/*!
 @brief Removes the VPN configuration from the Network Extension preferences if present.
 @discussion This method blocks a calling thread.
 @warning This method shouldn't be called from main thread.
 @return @c nil if removal was successful, error object otherwise.
 */
- (NSError *)removeProfile NS_AVAILABLE(10_7, 8_0);
/*!
 @brief Sets VPN configuration for current app enabled.
 @discussion If succeeded, this method disables VPN configurations of other apps. This method blocks a calling thread.
 @warning This method shouldn't be called from main thread.
 @return @c nil if VPN was enabled, error object otherwise.
 */
- (NSError *)enableVPN NS_AVAILABLE(10_11, 8_0);
/*!
 @brief Starts the VPN tunnel using the current VPN configuration.
 @discussion This method returns after VPN tunnel connection process is started and does not signals about the actual success of the VPN connection. In order to obtain the connection status, use the @c connectionStatus property. This method blocks a calling thread.
 @warning This method shouldn't be called from main thread.
 @see  @c connectionStatus
 @param error If method generates an error, a new error object is created and returned as an object pointed to by this parameter. Can be @c nil.
 @return YES if the VPN tunnel was started successfully, NO if an error occurred.
 */
- (BOOL)startVPNWithError:(NSError **)error NS_AVAILABLE(10_7, 8_0);
/*!
 @brief Stops the VPN tunnel.
 @discussion This method returns after VPN tunnel disconnect process is started and does not signals about the actual success of the VPN connection. In order to obtain the connection status, use the @c connectionStatus property. This method blocks a calling thread.
 @warning This method shouldn't be called from main thread.
 @see  @c connectionStatus
 @param error If method generates an error, a new error object is created and returned as an object pointed to by this parameter. Can be @c nil.
 */
- (void)stopVPNWithError:(NSError **)error NS_AVAILABLE(10_7, 8_0);
/*!
 @brief Loads VPN configuration preferences.
 @discussion This method loads the preferences for VPN configuration of the current application. You can call this method if you want to retrieve @c connectionStatus property before any call to configurator. This method blocks a calling thread.
 @warning This method shouldn't be called from main thread.
 @return @c nil if preferences were loaded successfully, error object otherwise.
 */
- (NSError *)loadPreferences NS_AVAILABLE(10_7, 8_0);
/*!
 @brief Turns VPN On Demand on or off.
 @discussion This method blocks a calling thread.
 @warning This method shouldn't be called from main thread.
 @param enabled Boolean value which controls VPN On Demand toggling. If set to @c YES VPN On Demand is turned on, otherwise VPN On Demand is turned off.
 @return @c nil on success, error object otherwise.
 */
- (NSError *)setOnDemandEnabled:(BOOL)enabled NS_AVAILABLE(10_7, 7_0);

/*!
 @brief Tells whether current platform version supports @c NEVPNManager.
 @discussion For macOS this method returns @c YES for OS version starting from 10.11+. Although @c NEVPNManager is available for 10.10, it just doesn't work.
 @return @c YES if current platform version supports @c NEVPNManager, @c NO otherwise.
 */
+ (BOOL)isNEVPNManagerSupportedByPlatform;

#pragma mark - Life Cycle

/*!
 @brief Initializes and returns a newly allocated VPN configurator object with the specified transport object.
 @param transport The transport object for internal usage.
 @return An initialized VPN configurator object or @c nil if the object couldn't be created.
 */
- (instancetype)initWithTransport:(VPNUTransport *)transport NS_AVAILABLE(10_7, 7_0) NS_DESIGNATED_INITIALIZER;

/*!
 @brief Creates and returns a newly allocated VPN configurator object with the specified transport object.
 @discussion This method is a convenience constructor.
 @param transport The transport object for internal usage.
 @return An initialized VPN configurator object or @c nil if the object couldn't be created.
 */
+ (instancetype)configuratorWithTransport:(VPNUTransport *)transport NS_AVAILABLE(10_7, 7_0);

@end
