//
//  VPNOnDemandRules.h
//  VPNUnlimitedSDK
//
//  Copyright (c) 2015 KeepSolid. All rights reserved.
//

//
// Use of this SDK is subject to the VPN Unlimited SDK License Agreement
// https://www.keepsolid.com/static/docs/VPN_Unlimited_SDK_License.html
//

#import <Foundation/Foundation.h>

/*!
 @class VPNUOnDemandConnectionManager
 @abstract
 This class provides a programmatic interface for managing VPN On Demand Rules.
 @discussion
 Use this class to manage Service Set Identifier (SSID) lists in conjunction with VPN On Demand rules. The instance of this class must be set as @c onDemandConnectionManager property of @c VPNUConfigurator if you want to apply these rules while setting up a VPN connection.
 
 This class automatically detects new SSIDs and sends a @c VPNUNewSSIDFoundNotification notification with newly detected SSID's string as a value for key @c kVPNUNewSSIDNotificationKey in @c userInfo dictionary of this notification.
 
 @attention You must reconfigure your current VPN connection with new settings each time you want them to take effect.
 */

NS_AVAILABLE(10_8, 8_0)
@interface VPNUOnDemandConnectionManager : NSObject <NSCoding>

/*!
 @abstract
 If set to @c YES, VPN connection auto-established when internet connection established. Default value is @c NO.
 */
@property (nonatomic, assign) BOOL onDemandEnabled NS_DEPRECATED_MAC(10_8, 10_11);

/*!
 @abstract
 If set to @c YES, VPN connection is allowed for cellular networks, disallowed otherwise. Default value is @c YES.
 */
@property (nonatomic, assign) BOOL connectOnCellular NS_AVAILABLE_IPHONE(8_0);

/*!
 @abstract
 If set to @c YES, VPN connection is allowed for wireless networks, disallowed otherwise. Default value is @c YES.
 */
@property (nonatomic, assign) BOOL connectOnWireless NS_AVAILABLE(10_11, 8_0);

/*!
 @abstract
 If set to @c YES, VPN connection is allowed for wired Ethernet connections, disallowed otherwise. Default value is @c YES.
 */
@property (nonatomic, assign) BOOL connectOnEthernet NS_AVAILABLE_MAC(10_11);

/*!
 @brief Adds the Service Set Identifier (SSID) to untrusted list.
 @param SSID String which contains a SSID. Cannot be nil.
 */
- (void)addSSIDToUntrusted:(NSString *)SSID NS_AVAILABLE(10_8, 8_0);

/*!
 @brief Adds the Service Set Identifier (SSID) to trusted list.
 @param SSID String which contains a SSID. Cannot be nil.
 */
- (void)addSSIDToTrusted:(NSString *)SSID NS_AVAILABLE(10_8, 8_0);

/*!
 @brief Removes the Service Set Identifier (SSID) from untrusted list.
 @param SSID String which contains a SSID. Cannot be nil.
 */
- (void)removeSSIDFromUntrusted:(NSString *)SSID NS_AVAILABLE(10_8, 8_0);

/*!
 @brief Removes the Service Set Identifier (SSID) from trusted list.
 @param SSID String which contains a SSID. Cannot be nil.
 */
- (void)removeSSIDFromTrusted:(NSString *)SSID NS_AVAILABLE(10_8, 8_0);

/*!
 @brief Removes the Service Set Identifier (SSID) from all lists.
 @param SSID String which contains a SSID. Cannot be nil.
 */
- (void)removeSSID:(NSString *)SSID NS_AVAILABLE(10_8, 8_0);

/*!
 @brief Gets a list of untrusted Service Set Identifiers (SSIDs).
 */
@property (readonly, copy) NSArray *SSIDUntrustedList NS_AVAILABLE(10_8, 8_0);

/*!
 @brief Gets a list of trusted Service Set Identifiers (SSIDs).
 */
@property (readonly, copy) NSArray *SSIDTrustedList NS_AVAILABLE(10_8, 8_0);

/*!
 @brief Gets a list of Service Set Identifiers (SSIDs).
 */
@property (readonly, copy) NSArray *SSIDList NS_AVAILABLE(10_8, 8_0);

@end
