//
//  VPNURequest.h
//  VPNUnlimitedSDK
//
//  Copyright (c) 2015 KeepSolid. All rights reserved.
//

//
// Use of this SDK is subject to the VPN Unlimited SDK License Agreement
// https://www.keepsolid.com/static/docs/VPN_Unlimited_SDK_License.html
//


#import <Foundation/Foundation.h>

/*!
 @class VPNURequest
 @abstract
 A special request class used by @c VPNUTransport.
 @discussion
 A request object is constructed with one of the factory methods. Most API requests should be constructed using @c APIRequestWithAction: or @c APIRequestWithAction:andParams: methods. The newly created request must then be passed as a parameter to @c sendRequest:error: or @c cURLSendRequest:error: methods of @c VPNUTransport class instance.
 */
NS_AVAILABLE(10_7, 7_0)
@interface VPNURequest : NSObject

#pragma mark - Properties

/*!
 @brief Indicates if the request should use current session. Default value is @c YES.
 */
@property (nonatomic) BOOL useSession NS_AVAILABLE(10_7, 7_0);

/*!
 @brief The timeout interval specifies the limit on the idle interval alloted to a request in the process of loading. Default value is 10.0.
 */
@property (nonatomic) NSTimeInterval timeoutInterval;

#pragma mark - Public Methods
/*!
 @brief Adds a given key-value pair to the parameters dictionary of the request.
 @param object The value for @c key. Must be of @c NSString or @c NSNumber class. Must not be @c nil.
 @param key The key for value. Must not be @c nil.
 */
- (void)setParameterObject:(id)object forKey:(NSString *)key NS_AVAILABLE(10_7, 7_0);
/*!
 @brief Returns the value object associated with a given key in the parameters dictionary of the request.
 @param key The key for which to return the corresponding object value. Must not be @c nil.
 @return The value object associated with @c key in the parameters dictionary of the request, or @c nil if no object is associated with @c key.
 */
- (id)parameterObjectForKey:(NSString *)key NS_AVAILABLE(10_7, 7_0);

#pragma mark - Class Methods

/*!
 @brief Sets the application ID needed for request signing
 @param appID Your application ID.
*/
+ (void)setApplicationID:(NSString *)appID;
/*!
 @brief Sets the application secret key needed for request signing
 @param appSecret Your application secret key.
 */
+ (void)setApplicationSecret:(NSString *)appSecret;

/*!
 @brief Explicitly indicate bundle ID.
 @param Your application bundle ID.
 */
+ (void)setApplicationBundleID:(NSString *)bundleID;

#pragma mark - Factory Methods
/*!
 @brief Same as @c authRequestWithAction:andParams: but does not use any extra parameters.
 */
+ (instancetype)authRequestWithAction:(NSString *)action NS_AVAILABLE(10_7, 7_0);
/*!
 @brief Creates and returns a newly constructed @c VPNURequest object for authorization-related routines using a specified action.
 @param action One of the predefined action strings associated with the new request. Cannot be @c nil.
 @param params A dictionary containing additional parameters for the request. Can be @c nil.
 @return The newly constructed @c VPNURequest object or @c nil if construction fails.
 */
+ (instancetype)authRequestWithAction:(NSString *)action andParams:(NSDictionary *)params NS_AVAILABLE(10_7, 7_0);

/*!
 @brief Same as @c APIRequestWithAction:andParams: but does not use any extra parameters.
 */
+ (instancetype)APIRequestWithAction:(NSString *)action NS_AVAILABLE(10_7, 7_0);
/*!
 @brief Creates and returns a newly constructed @c VPNURequest object for API-related routines using a specified action.
 @param action One of the predefined action strings associated with the new request. Cannot be @c nil.
 @param params A dictionary containing additional parameters for the request. Can be @c nil.
 @return The newly constructed @c VPNURequest object or @c nil if construction fails.
 */
+ (instancetype)APIRequestWithAction:(NSString *)action andParams:(NSDictionary *)params NS_AVAILABLE(10_7, 7_0);

@end
