//
//  VPNUStorage.h
//  VPNUnlimitedSDK
//
//  Copyright (c) 2015 KeepSolid. All rights reserved.
//

//
// Use of this SDK is subject to the VPN Unlimited SDK License Agreement
// https://www.keepsolid.com/static/docs/VPN_Unlimited_SDK_License.html
//

#import <Foundation/Foundation.h>

@class VPNUAccountUserInfo;

/*!
 @class VPNUStorage
 @abstract
 Special class which can be used to save account-specific information on disk.
 @discussion Use this class if you want to save any information which is specific for an account described by @c VPNUAccountUserInfo. This class also provides an interface to save data securely using methods which use encryption and decryption.
 
 @attention The class uses @c NSUserDefaults internally. If you clear the defaults associated with the intance of this class (generally @c standardUserDefaults), all stored values (including encrypted) will be lost.
 */
NS_AVAILABLE(10_7, 7_0)
@interface VPNUStorage : NSObject

/*!
 @brief Encrypts and stores provided key-value pair (key and object) for the specified account.
 @warning This method generates an exception if no encryption key was provided using @c setEncryptionKey: method.
 @warning The object parameter can be only property list objects: NSData, NSString, NSNumber, NSDate, NSArray, or NSDictionary.
 @param object The object to store. Cannot be nil.
 @param key The key with which to associate with the object. Cannot be nil.
 @param account The account with which to associate with the given key-value pair.
 */
- (void)storeWithEncryptionObject:(id)object forKey:(NSString *)key forAccount:(VPNUAccountUserInfo *)account NS_AVAILABLE(10_7, 7_0);
/*!
 @brief Decrypts and returns the object associated with the provided key and account.
 @warning This method generates an exception if no encryption key was provided using @c setEncryptionKey: method.
 @param key The key with which to associate with the object. Cannot be nil.
 @param account The account with which to associate with the given key.
 */
- (id)readWithDecryptionObjectForKey:(NSString *)key forAccount:(VPNUAccountUserInfo *)account NS_AVAILABLE(10_7, 7_0);

/*!
 @brief Stores provided key-value pair (key and object) for the specified account.
 @warning The object parameter can be only property list objects: NSData, NSString, NSNumber, NSDate, NSArray, or NSDictionary.
 @param object The object to store.
 @param key The key with which to associate with the object. Cannot be nil.
 @param account The account with which to associate with the given key-value pair.
 */
- (void)storeObject:(id)object forKey:(NSString *)key forAccount:(VPNUAccountUserInfo *)account NS_AVAILABLE(10_7, 7_0);
/*!
 @brief Returns the object associated with the provided key and account.
 @param key The key with which to associate with the object. Cannot be nil.
 @param account The account with which to associate with the given key.
 */
- (id)readObjectForKey:(NSString *)key forAccount:(VPNUAccountUserInfo *)account NS_AVAILABLE(10_7, 7_0);


/*!
 @brief Encrypts and stores provided object for the specified key.
 @warning This method generates an exception if no encryption key was provided using @c setEncryptionKey: method.
 @warning The object parameter can be only property list objects: NSData, NSString, NSNumber, NSDate, NSArray, or NSDictionary.
 @param object The object to store. Cannot be nil.
 @param key The key with which to associate with the object. Cannot be nil.
 */
- (void)storeWithEncryptionObject:(id)object forKey:(NSString *)key NS_AVAILABLE(10_7, 7_0);
/*!
 @brief Decrypts and returns the object associated with the provided key.
 @warning This method generates an exception if no encryption key was provided using @c setEncryptionKey: method.
 @param key The key with which to associate with the object. Cannot be nil.
 */
- (id)readWithDecryptionObjectForKey:(NSString *)key NS_AVAILABLE(10_7, 7_0);
/*!
 @brief Stores provided object for the specified key.
 @warning The object parameter can be only property list objects: NSData, NSString, NSNumber, NSDate, NSArray, or NSDictionary.
 @param object The object to store.
 @param key The key with which to associate with the object. Cannot be nil.
 */
- (void)storeObject:(id)object forKey:(NSString *)key NS_AVAILABLE(10_7, 7_0);
/*!
 @brief Returns the object associated with the provided key.
 @param key The key with which to associate with the object. Cannot be nil.
 */
- (id)readObjectForKey:(NSString *)key NS_AVAILABLE(10_7, 7_0);


/*!
 @brief Sets the provided string as the encryption and decryption key used by secure methods.
 @discussion The provided string is later used internally when the methods with pattern @c storeWithEncryptionObject: and @c readWithDecryptionObjectForKey: are called. It is recommended to use an arbitrary, comparatively large and permutated string for this value. Generally you are discouraged to use a user's login or password or any meaningful string as an encryption key.
 
 @attention Set the key only once before making any calls to the instance. Otherwise the correctness of encryption and decryption is not guaranteed.
 
 @attention Encryption key should be the same between the launches of your host application. If you change the key, all previously saved values will be inaccessible until you set the key back to the previous value.
 
 @param key Encryption key to be used for encryption. Cannot be nil.
 */
- (void)setEncryptionKey:(NSString *)key NS_AVAILABLE(10_7, 7_0);

/*!
 @brief Returns the shared storage object.
 @discussion The shared storage is created internally with @c standardUserDefaults instance of @c NSUserDefaults class. If you want to use any custom @c NSUserDefaults object, you can initialize your own instance of @c VPNUStorage using @c initWithUserDefaults: covinience initializer.
 */
+ (instancetype)defaultStorage NS_AVAILABLE(10_7, 7_0);
/*!
 @brief Initializes and return a newly created @c VPNUStorage instance which uses a custom @c NSUserDefaults object.
 @discussion This method is a convenience constructor.
 
 @attention Use this method only if you know what you are doing. Generally you want to use @c standardUserDefaults instance of @c NSUserDefaults class - simply use the @c defaultStorage class method (the latter is generally preferred).
 
 @param userDefaults A custom @c NSUserDefaults object.
 */
- (instancetype)initWithUserDefaults:(NSUserDefaults *)userDefaults NS_DESIGNATED_INITIALIZER;

@end
